"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var template_1 = tslib_1.__importDefault(require("@babel/template"));
var px2vw_1 = require("./px2vw");
var types_1 = require("@babel/types");
var configuration_1 = tslib_1.__importDefault(require("./configuration"));
var replace_1 = require("./replace");
function isStyledTagged(tagged) {
    var tag = tagged.tag;
    if (types_1.isIdentifier(tag)) {
        return isStyled(tag);
    }
    else if (types_1.isMemberExpression(tag) && types_1.isIdentifier(tag.object)) {
        return isStyled(tag.object);
    }
    else if (types_1.isCallExpression(tag)) {
        if (types_1.isIdentifier(tag.callee)) {
            return isStyled(tag.callee);
        }
        else if (types_1.isMemberExpression(tag.callee)) {
            return isStyledMember(tag.callee);
        }
    }
    return false;
}
function isStyledMember(member) {
    if (types_1.isIdentifier(member.object)) {
        return isStyled(member.object);
    }
    else if (types_1.isMemberExpression(member.object)) {
        return isStyledMember(member.object);
    }
    return false;
}
function isStyled(id) {
    return configuration_1.default.config.tags.indexOf(id.name) >= 0;
}
function transformTemplateElement(it) {
    if (it.value && it.value.raw) {
        it.value.raw = replace_1.replace(it.value.raw);
    }
    if (it.value && it.value.cooked) {
        it.value.cooked = replace_1.replace(it.value.cooked);
    }
}
function transformTemplateExpression(expression, px2vw) {
    if (types_1.isArrowFunctionExpression(expression)) {
        if (types_1.isBlock(expression.body)) {
            expression.body = types_1.callExpression(px2vw, [types_1.arrowFunctionExpression([], expression.body)]);
        }
        else {
            expression.body = types_1.callExpression(px2vw, [expression.body]);
        }
    }
    else if (types_1.isConditionalExpression(expression)) {
        expression.consequent = transformTemplateExpression(expression.consequent, px2vw);
        expression.alternate = transformTemplateExpression(expression.alternate, px2vw);
    }
    else {
        return types_1.callExpression(px2vw, [expression]);
    }
    return expression;
}
var _px2vw;
function transform(template) {
    var _a, _b;
    if (!template.expressions || template.expressions.length === 0) {
        template.quasis.forEach(transformTemplateElement);
    }
    else {
        var expressions = tslib_1.__spreadArray(tslib_1.__spreadArray([], template.expressions), template.quasis);
        expressions.sort(function (it1, it2) { return (it1.start || 0) - (it2.start || 0); });
        var _loop_1 = function (i) {
            var expression = expressions[i];
            if (types_1.isTemplateElement(expression)) {
                transformTemplateElement(expression);
            }
            else if (_px2vw) {
                var next = expressions[i + 1];
                if (next && types_1.isTemplateElement(next)) {
                    var text = ((_a = next.value) === null || _a === void 0 ? void 0 : _a.raw) || ((_b = next.value) === null || _b === void 0 ? void 0 : _b.cooked);
                    if (text && /^px/.test(text)) {
                        var idx = template.expressions.findIndex(function (it) { return it === expression; });
                        if (idx !== -1) {
                            template.expressions[idx] = transformTemplateExpression(expression, _px2vw);
                        }
                        if (next.value && next.value.raw) {
                            next.value.raw = next.value.raw.replace(/^px/, '');
                        }
                        if (next.value && next.value.cooked) {
                            next.value.cooked = next.value.cooked.replace(/^px/, '');
                        }
                    }
                }
            }
        };
        for (var i = 0; i < expressions.length; i++) {
            _loop_1(i);
        }
    }
}
exports.default = helper_plugin_utils_1.declare(function (api, options) {
    api.assertVersion(7);
    configuration_1.default.updateConfig(options);
    var templateVisitor = {
        TemplateLiteral: {
            enter: function (templateLiteralPath) {
                transform(templateLiteralPath.node);
            },
        },
    };
    var visitor = {
        Program: {
            exit: function () {
                _px2vw = undefined;
            },
            enter: function (programPath) {
                if (configuration_1.default.config.transformRuntime) {
                    _px2vw = programPath.scope.generateUidIdentifier('px2vw');
                    var template = template_1.default.statement(px2vw_1.px2vw);
                    programPath.node.body.push(template({
                        input: types_1.identifier('input'),
                        px2vw: _px2vw,
                        viewportWidth: types_1.numericLiteral(configuration_1.default.config.viewportWidth),
                        unitPrecision: types_1.numericLiteral(configuration_1.default.config.unitPrecision),
                        minPixelValue: types_1.numericLiteral(configuration_1.default.config.minPixelValue),
                    }));
                }
                programPath.traverse({
                    TaggedTemplateExpression: function (path) {
                        if (isStyledTagged(path.node)) {
                            path.traverse(templateVisitor);
                        }
                    },
                    CallExpression: function (path) {
                        if (types_1.isMemberExpression(path.node.callee) &&
                            types_1.isIdentifier(path.node.callee.object) &&
                            isStyled(path.node.callee.object)) {
                            path.traverse(templateVisitor);
                        }
                    },
                });
            },
        },
    };
    return {
        name: 'styled-components-px2vw',
        visitor: visitor,
    };
});
